import { foldNodeProp, indentNodeProp, LezerSyntax } from "@codemirror/next/syntax";
import { styleTags } from "@codemirror/next/highlight";
import { parser } from "lezer-gcode";
/// A syntax provider based on the Lezer GCODE
/// parser, extended with
/// highlighting and indentation information.
export const gcodeSyntax = LezerSyntax.define(parser.withProps(indentNodeProp.add(type => {
    // if (type.name == "Loop_Body") return continuedIndent()
    if (type.name == 'End_Loop')
        return context => context.column(context.pos) - context.baseIndent;
    if (type.name == "Do_Loop")
        return context => context.baseIndent + context.unit;
    return undefined;
}), foldNodeProp.add({
    While_Loop(subtree) {
        var _a;
        // @ts-ignore
        let endLoopLength;
        (_a = subtree.childBefore(subtree.lastChild.end)) === null || _a === void 0 ? void 0 : _a.iterate({
            enter: (type, start, end) => {
                if ('Expression' === type.name) {
                    endLoopLength = end - start;
                }
            }
        });
        return { from: subtree.start + 5, to: subtree.lastChild.end - endLoopLength };
    }
}), styleTags({
    COMMENT: "comment",
    "FLOAT INT": "atom",
    PROGRAM_NUMBER: "number",
    //keywords
    "AND BCD BIN DO END GOTO IF OR THEN WHILE XOR": "keyword definition",
    "ABS ACOS ASIN ATAN COS FIX FUP ROUND SIN SQRT TAN": "keyword#2 definition",
    "LT GT LE GE NE EQ": "operatorKeyword",
    "A B C D E F G H I J K L M N O P Q R S T U V W X Y Z": "strong name#2",
    "+ - * / = #": "meta arithmeticOperator",
    "#": "meta definition",
    "( )": "meta paren",
    "[ ]": "meta bracket",
    "{ }": "meta brace"
})), {
    languageData: {
        commentTokens: { line: { open: "(", close: ")" } },
    },
});
/// Returns an extension that installs the gcode syntax and
/// support features.
export function gcode() {
    return [gcodeSyntax];
}
//# sourceMappingURL=gcode.js.map